import java.awt.Point;
import java.util.Random;
import java.util.Scanner;




/**
 * Jogador que tenta pegar bolas arremessadas pelos adversários e que joga bolas neles também
 * 
 * @author Equipe Baixo Nível
 * @since 16/03/2010
 * @version 0.1
 * 
 * 2010-04-16 - Corrigida a lógica de parar com as tentativas de pegar bolas lançadas pelos 
 * 				adversários. Agora verifico se tentou pegar e o conteúdo das mãos não mudou. 
 * 
 * 				Melhorando a escolha do movimento. Dica do jogador exemplo em C++, camper.
 * 				Marca o local para onde a criança está indo para reduzir # de conflitos.
 * 				Marca o local de onde pegou neve para reduzir # de conflitos.
 * 
 * 				Testando a idéia de roubar bonecos de neve. Deu muito certo!
 * 
 * 
 */
public class Sucker {

	// Constant used to mark child locations in the map.
	public static final int GROUND_CHILD = 10;
	// Another extra constant to mark spaces we should not pick up from.
	public static final int GROUND_TAKEN = 11;
	
	public static final int GROUND_SMBT = 12;
	
	public static final int GROUND_DONT_DESTROY_MY_BALL = 13;

	/** Current game score for self (red) and opponent (blue). */
	private int[] score = new int[2];

	/** Current snow height in each cell. */
	private int[][] height = new int[Const.SIZE][Const.SIZE];

	/** Contents of each cell. */
	private int[][] ground = new int[Const.SIZE][Const.SIZE];
	// Conteúdo antigo das células
	private int[][] ground_old = new int[Const.SIZE][Const.SIZE];
	
	/** List of children on the field, half for each team. */
	private Child[] cList = new Child[2 * Const.CCOUNT];

	/** Turno do jogo */
	private int turnNum;
	
	/** Source of randomness for this player. */
	static Random rnd = new Random();

	/** Return the value of x, clamped to the [ a, b ] range. */
	static int clamp(int x, int a, int b) {
		if (x < a)
			return a;
		if (x > b)
			return b;
		return x;
	}

	// Simple representation for a child's action
	public class Move {
		/** Make an idle move. */
		Move() {
			action = "idle";
		}

		/** Make a move for the given action. */
		Move(String act) {
			action = act;
		}

		/** Make a move for the given action and destination. */
		Move(String act, int x, int y) {
			action = act;
			dest = new Point(x, y);
		}

		// Action the child is making.
		String action = "idle";

		// Destiantion of this action (or null, if it doesn't need one) */
		Point dest = null;
	}

	// Simple representation for a child in the game. Subclasses of
	// child can implement their own behavior.
	public class Child {
		
		// ID da criança
		int id;
		
		// Location of the child.
		Point pos = new Point();

		// True if the child is standing.
		boolean standing;

		// Side the child is on.
		int color;

		// What's the child holding now.
		int holding;
		// O que ela tinha na rodada anterior
		int holding_old;
		// O último movimento que ela fez
		String moveOld = "";
		
		// How many more turns this child is dazed.
		int dazed;

		/**
		 * The child object provides both behavior and representation. We don't
		 * need the behavior part for snowmen on the opposing team, so we just
		 * make instances of the superclass for them.
		 */
		public Move chooseMove() {
			return new Move();
		}
	}

	/**
	 * sequence of moves templates to build a to the right of the player. For
	 * the first one, we're just looking for a place to build.
	 */
	public Move[] instructions = { 
		new Move("idle"), 
		new Move("crouch"),
		new Move("pickup", 1, 0), 
		new Move("pickup", 1, 0),
		new Move("pickup", 1, 0), 
		new Move("crush"),
		new Move("drop", 1, 0), 
		new Move("pickup", 1, 1),
		new Move("pickup", 1, 1), 
		new Move("crush"),
		new Move("drop", 1, 0), 
		new Move("pickup", 1, 1),
		new Move("crush"), 
		new Move("drop", 1, 0), 
		new Move("stand"), 
	};

	// Child that moves away from other children and
	// then builds a snowman.
	public class SnowmanMaker extends Child {
		/** Current instruction this child is executing. */
		int state = 0;

		int catchFailCounter = 0;
		boolean tryCatch = true;
		
		boolean blueSnowManDown = false;
		Point newSnowManPoint = null;
		
		/** Current destination of this child. */
		Point runTarget = new Point();

		/**
		 * How many more turns is this child going to run toward the target.
		 */
		int runTimer;

		/** Return a move to get this child closer to target. */
		// Adicionada verificação de existência de obstáculos no caminho
		Move moveToward(Point target) {
			if (standing) {
				// Tenta se mover na diagonal ou duas casas por vez
				// Run to the destination
				if (pos.x != target.x) {
					if (pos.y != target.y) {
						// Run diagonally.
						if (groundFreeToMove(pos.x + clamp(target.x - pos.x, -1, 1), 
									   		 pos.y + clamp(target.y - pos.y, -1, 1))) 
							return new Move("run", pos.x + clamp(target.x - pos.x, -1, 1), 
												   pos.y + clamp(target.y - pos.y, -1, 1));
					}
					// Run left or right
					if (groundFreeToMove(pos.x + clamp(target.x - pos.x, -2, 2), 
							   			 pos.y)) 
						return new Move("run", pos.x + clamp(target.x - pos.x, -2, 2), 
										   	   pos.y);
				}

				if (pos.y != target.y)
					// Run up or down.
					if (groundFreeToMove(pos.x, 
							   			 pos.y + clamp(target.y - pos.y, -2, 2))) 
					return new Move("run", pos.x, 
										   pos.y + clamp(target.y - pos.y, -2, 2));
				
				// Se ainda n�o andou, tenta se mover uma casa por vez
				if (pos.x != target.x){
					// crawl left or right
					if (groundFreeToMove(pos.x + clamp(target.x - pos.x, -1, 1), 
						 	 			 pos.y)) 
						return new Move("run", pos.x + clamp(target.x - pos.x, -1, 1), 
											   pos.y);
				//se chegar até verificar se há obstáculos imediatamente do ladoxxx
					else if (groundFreeToMove(pos.x + clamp(target.x - pos.x, -1, 1), 
			 	 			 pos.y-1)) 
						return new Move("run", pos.x + clamp(target.x - pos.x, -1, 1), 
								   pos.y-1);
					else if (groundFreeToMove(pos.x + clamp(target.x - pos.x, -1, 1), 
			 	 			 pos.y+1)) 
						return new Move("run", pos.x + clamp(target.x - pos.x, -1, 1), 
								   pos.y+1);
				}
				if (pos.y != target.y){
					// crawl up or down.
					if (groundFreeToMove(pos.x, 
						 	 			 pos.y + clamp(target.y - pos.y, -1, 1))) 
						return new Move("run", pos.x, 
											   pos.y + clamp(target.y - pos.y, -1, 1));
				//se chegar até verificar se há obstáculos imediatamente acima ou abaixoxxx
					else if (groundFreeToMove(pos.x-1, 
										pos.y + clamp(target.y - pos.y, -1, 1))) 
						return new Move("run", pos.x-1, 
												pos.y + clamp(target.y - pos.y, -1, 1));
					else if (groundFreeToMove(pos.x+1, 
										pos.y + clamp(target.y - pos.y, -1, 1))) 
						return new Move("run", pos.x+1, 
									pos.y + clamp(target.y - pos.y, -1, 1));
				}
				
			} else {
			
				// Crawl to the destination
				if (pos.x != target.x){
					// crawl left or right
					if (groundFreeToMove(pos.x + clamp(target.x - pos.x, -1, 1), 
						 	 			 pos.y)) 
						return new Move("crawl", pos.x + clamp(target.x - pos.x, -1, 1), 
											 	 pos.y);
				}
				if (pos.y != target.y){
					// crawl up or down.
					if (groundFreeToMove(pos.x, 
						 	 			 pos.y + clamp(target.y - pos.y, -1, 1))) 
						return new Move("crawl", pos.x, 
											 	 pos.y + clamp(target.y - pos.y, -1, 1));
				}
				if (pos.x != target.x) {
					//se chegar até verificar se há obstáculos imediatamente do ladoxxx
					//obs agachado não se move na diagonal
						 if (groundFreeToMove(pos.x, pos.y-1)) 
							return new Move("crawl", pos.x, pos.y-1);
						 if (groundFreeToMove(pos.x, pos.y+1)) 
							return new Move("crawl", pos.x,pos.y+1);
				}
				if (pos.y != target.y) {
					//se chegar até verificar se há obstáculos imediatamente acima ou abaixoxxx
					//obs agachado não se move na diagonal
						if (groundFreeToMove(pos.x-1,pos.y))
							return new Move("crawl", pos.x-1,pos.y);
						if (groundFreeToMove(pos.x+1, pos.y)) 
							return new Move("crawl", pos.x+1, pos.y);
				}
			}			

			// Nowhere to move, just return the idle move.
			return new Move();
		}

		public boolean groundFreeToMove(int x, int y) {
			if (ground[x][y] != Const.GROUND_TREE &&	// Se não tem uma árvore
				ground[x][y] != Const.GROUND_SMR &&		// nem um boneco de neve vermelho
				ground[x][y] != Const.GROUND_SMB &&		// nem um boneco de neve azul
				ground[x][y] != GROUND_CHILD &&			// nem uma criança
				ground[x][y] != GROUND_DONT_DESTROY_MY_BALL &&	 // nem uma bola de um boneco semiconstruido
				getHeight(x, y) < 6) {					// e a altura de neve é menor que 6
			
				// Update the path as well
				Point[] path = childPath(new Point(x, y));
				// Se path tem posicao intermediaria, marca ela como ocupada para outra crianca nao tentar passar
				// por ali
				if (path.length == 2) {		// path: posicao intermediaria, posicao final
					int ix = path[0].x;		// x da posicao intermediaria
					int iy = path[0].y;		// y da posicao intermediaria
					
					if (ground[ix][iy] != Const.GROUND_TREE &&
						ground[ix][iy] != Const.GROUND_SMR &&
						ground[ix][iy] != Const.GROUND_SMB &&
						ground[ix][iy] != GROUND_CHILD &&
						getHeight(ix, iy) < 6) {
						
						// Update the map, so that the next child won't try to move right into this one.
						ground[x][y] = GROUND_CHILD;
						
						// a posicao intermediaria esta livre
						// marcamos ela como ocupada
						//System.err.println("posicao intermediaria: " + path[0].x + " " + path[0].y);
						ground[path[0].x][path[0].y] = GROUND_CHILD;
						
						return true;
					}
					else {
						// posicao final livre, mas intermediaria nao
						return false;
					}
				}
				// posicao final livre, nao ha posicao intermediaria
				ground[x][y] = GROUND_CHILD;
				return true;
				
			}
			else {
				// posicao final nao esta livre, mas pode ser que exista uma posicao intermediaria livre
				// se tiver, vamos andar ate ela
				// Update the path as well
				Point[] path = childPath(new Point(x, y));
				// Se path tem posicao intermediaria, marca ela como ocupada para outra crianca nao tentar passar
				// por ali
				if (path.length == 2) {		// path: posicao intermediaria, posicao final
					int ix = path[0].x;		// x da posicao intermediaria
					int iy = path[0].y;		// y da posicao intermediaria
					
					if (ground[ix][iy] != Const.GROUND_TREE &&
						ground[ix][iy] != Const.GROUND_SMR &&
						ground[ix][iy] != Const.GROUND_SMB &&
						ground[ix][iy] != GROUND_CHILD &&
						ground[x][y] != GROUND_DONT_DESTROY_MY_BALL &&
						getHeight(ix, iy) < 6) {
						
						// a posicao intermediaria esta livre
						// marcamos ela como ocupada
						//System.err.println("posicao intermediaria: " + path[0].x + " " + path[0].y);
						ground[path[0].x][path[0].y] = GROUND_CHILD;
						
						return true;
					}
					else {
						// nem posicao final nem intermediaria estao livres
						return false;
					}
				}
				// posicao final nao esta livre, nao ha posicao intermediaria
				return false;
			}
		}
		
		public int getHeight(int x, int y) {
			int h = 0;
			switch (ground[x][y]) {
			case Const.GROUND_EMPTY:
				h = 0;
				break;
			case Const.GROUND_S:
				h = 1;
				break;
			case Const.GROUND_M:
				h = 2;
				break;
			case Const.GROUND_MS:
				h = 3;
				break;
			case Const.GROUND_L:
				h = 3;
				break;
			case Const.GROUND_LM:
				h = 5;
				break;
			case Const.GROUND_LS:
				h = 4;
				break;
			case Const.GROUND_SMR:
				h = 6;
				break;
			case Const.GROUND_SMB:
				h = 6;
				break;
			case Const.GROUND_TREE:
				h = 9;
				break;
			case GROUND_CHILD:
				h = 9;
				break;
			}
			
			return h + height[x][y];
		}
		
		/** Calcula as posicoes pelas quais a bola ira passar em sua missao de destruicao (game.pdf, p. 7) */
		public Point[] ballPath(Point target) {
			// numero de passos da trajetoria
			int n = Math.max(Math.abs(pos.x - target.x), Math.abs(pos.y - target.y));
			Point[] path = new Point[n];
			for (int i=0; i<n; i++) {
				int t = i + 1;
				int x = pos.x + round((double)(t*(target.x - pos.x)) / n);
				int y = pos.y + round((double)(t*(target.y - pos.y)) / n);
				path[i] = new Point(x, y);
			}
			return path;
		}
		
		/** Funcao de arredondamento (game.pdf, p. 8) */
		public int round(double x) {
			if (x<0) {
				return -(int)(Math.round(-x));
			}
			return (int)(Math.round(x));
		}
		
		public void printPath(Point[] path) {
			for (int i=0; i < path.length; i++) {
				System.err.println("path[" + i + "]:" + path[i].x + " " + path[i].y);
			}
		}
		
		/** Testa se ha um obstaculo na trajetoria da bola.*/
		public boolean obstacleInPath(Point victim, Point target) {
			Point[] path = ballPath(target);
			
			for (int i=0; i < path.length; i++) {
				int x = path[i].x;
				int y = path[i].y;
				if (x == victim.x && y == victim.y) {
					// bola ja chegou na vitima, nao encontramos nenhum obstaculo
					return false;
				}
				int space = ground[x][y];
				if (space == Const.GROUND_TREE) {
					return true;
				}
				if (space == GROUND_CHILD) { 	// se tiver uma crianca no meio do caminho
				
					for (int j = 0; j < Const.CCOUNT; j++) {
						if (cList[j].pos.x == x && cList[j].pos.y == y) {		// a crianca eh vermelha
							return true;
						}
					}
					// a crianca eh azul, nao tem problema acertar ela entao consideramos que nao eh um obstaculo
					return false;
				}
			}
			return false;
		}
		
		/** Calcula as posicoes pelas quais a crianca ira passar (game.pdf, p. 7) */
		public Point[] childPath(Point target) {
			// numero de passos da trajetoria
			int n = Math.max(Math.abs(pos.x - target.x), Math.abs(pos.y - target.y));
			Point[] path = new Point[n];
			for (int i=0; i<n; i++) {
				int t = i + 1;
				int x = pos.x + round((double)(t*(target.x - pos.x)) / n);
				int y = pos.y + round((double)(t*(target.y - pos.y)) / n);
				path[i] = new Point(x, y);
			}
			return path;
		}
		
		/** Testa se ha uma obstaculo entre a crianca e seu destino */
		public boolean obstacleInChildsPath(Point target) {
			Point[] path = childPath(target);

			for (int i=0; i < path.length; i++) {
				int x = path[i].x;
				int y = path[i].y;
				int space = ground[x][y];
				if (space == Const.GROUND_TREE) {
					return true;			// ha uma arvore no meio do caminho
				}
				if (space == GROUND_CHILD) { 	
					return true;			// ha uma crianca no meio do caminho
				}
			}
			return false;					// nao ha obstaculos
		}
		
		public Move chooseMove() {
			
			// TODO testar isso aqui!
			// Se esta no comeco do jogo, faz um boneco com uma das criancas perto do comeco
			if (id == 1 && turnNum < 19) {
				Point snowmanPos = new Point(7,7);		// vou construir um boneco aqui
				// marcamos o chao para outros nao passarem por cima
				ground[snowmanPos.x][snowmanPos.y] = GROUND_DONT_DESTROY_MY_BALL;		
				Point destination = new Point(6,7);		// a crianca vai construir o boneco daqui
				if (turnNum == 0) {
					state = 0;		// sinalizo que vou andar ate o destino
				}
				if (state == 0) {
					// verifica se ja chegou no destino
					if (pos.x == destination.x && pos.y == destination.y) {
						// ja chegou no destino, sinaliza que deve construir o boneco
						state = 1;
					}
					else {
						// ainda nao cheguei no destino
						return moveToward(destination);
					}
					
				}
				// Are we building a snowman?
				if (state > 0)  {
					// Stamp out a move from our instruction template and return it.
					Move m = new Move( instructions[ state ].action );
					if ( instructions[ state ].dest != null )
						m.dest = new Point( pos.x + instructions[ state ].dest.x,
								pos.y + instructions[ state ].dest.y );
					state = ( state + 1 ) % instructions.length;

					return m;
				}
					
			}
			// Se esta no comeco do jogo, faz um boneco com uma das criancas perto do comeco
			if (id == 3 && turnNum < 25) {
				Point snowmanPos = new Point(21,7);		// vou construir um boneco aqui
				// marcamos o chao para outros nao passarem por cima
				ground[snowmanPos.x][snowmanPos.y] = GROUND_DONT_DESTROY_MY_BALL;		
				Point destination = new Point(20,7);		// a crianca vai construir o boneco daqui
				if (turnNum == 0) {
					state = 0;		// sinalizo que vou andar ate o destino
				}
				if (state == 0) {
					// verifica se ja chegou no destino
					if (pos.x == destination.x && pos.y == destination.y) {
						// ja chegou no destino, sinaliza que deve construir o boneco
						state = 1;
					}
					else {
						// ainda nao cheguei no destino
						return moveToward(destination);
					}
					
				}
				// Are we building a snowman?
				if (state > 0)  {
					// Stamp out a move from our instruction template and return it.
					Move m = new Move( instructions[ state ].action );
					if ( instructions[ state ].dest != null )
						m.dest = new Point( pos.x + instructions[ state ].dest.x,
								pos.y + instructions[ state ].dest.y );
					state = ( state + 1 ) % instructions.length;

					return m;
				}
					
			}
			//defende 2
			if (id == 2 && turnNum < 25) {
				runTarget.setLocation(20, 10);
				return moveToward(runTarget);
					
			}			
					
			// Se está tonto, não faz nada
			if (dazed > 0) {
				// Se levei uma bolada, volto a tentar pegar bolas
				tryCatch = true;
				catchFailCounter = 0;
				return new Move();
			}

			// Reconstruir boneco tem prioridade maior que atacar ou interceptar bola
			// Se acabou de destruir um boneco de neve azul, refaz como vermelho
			if (blueSnowManDown) {
				blueSnowManDown = false;
				return new Move("drop",newSnowManPoint.x,newSnowManPoint.y);
			}
			
			// Verifica se deve continuar tentando pegar bolas
			// Se tentei pegar uma bola na rodada anterior
			if (moveOld.compareTo("catch") == 0) {
				// Se não mudou o conteúdo que tinha nas mãos
				if (holding == holding_old) {
					// Incremento contador de tentativas falhas
					catchFailCounter++;
					// E vejo se devo parar de tentar pegar
					if (catchFailCounter >= 2)
						tryCatch = false;
				} else {
					// Se peguei uma bola, continuo tentando pegar bolas
					catchFailCounter = 0;
					tryCatch = true;
				}
			}
				
			// Primeiro tenta pegar bolas atiradas pelo adversário
			// Se não estou com muitas bolas de neve na mão, então posso pegar
			if (tryCatch && (holding == Const.HOLD_EMPTY || holding == Const.HOLD_S1 || holding == Const.HOLD_S2)) {
				// Para cada criança adversária
				for (int j = Const.CCOUNT; j < Const.CCOUNT * 2; j++) {
					Child c = cList[j];
					// Se conheço a sua posição e 
					// você tem ao menos uma bola de neve na mão e 
					// você não está tonto
					if (c.pos.x >= 0 &&	
						(c.holding == Const.HOLD_S1 || c.holding == Const.HOLD_S2 || c.holding == Const.HOLD_S3) &&
						c.dazed <= 0) {
						int dx = c.pos.x - pos.x;
						int dy = c.pos.y - pos.y;
						// Calculo a distância entre nós
						int dsq = dx * dx + dy * dy;
						// Se pode me alcançar 
						if (dsq < 8 * 8) {
							return new Move("catch", c.pos.x, c.pos.y);
						}
					}
				}
			}
			
			// TODO TESTAR Procura por um boneco de neve azul (adversário) para tentar destruí-lo
			boolean smb = false;
			Point po = null;
			int dist = 99999;
			for (int i = 0; i < Const.SIZE; i++) {
				for (int j = 0; j < Const.SIZE; j++) {
					// Se encontrei a posição de um boneco de neve azul
					if (ground[i][j] == Const.GROUND_SMB) {
						int dx = i - pos.x;
						int dy = j - pos.y;
						int dsq = dx * dx + dy * dy;
						
						smb = true;
							
						// Se está em pé, perto do boneco e tem pelo menos uma bola de neve nas mãos 
// Desistir de destruir pois perco tempo fazendo outra bola para colocar no topo
// Melhor mesmo é pegar a bola do topo e recolocá-la
//						if (standing && dsq < 6 * 6 && dsq > 4 * 4 &&
//							(holding == Const.HOLD_S1 || holding == Const.HOLD_S2 || holding == Const.HOLD_S3)) {
							// throw past (tree times) the snowman, so we will probably
							// hit them in the top.
//							return new Move("throw", pos.x + dx * 3, pos.y + dy * 3);
//						} else {
						if (dsq <= 2) {
							// Estou do lado do boneco, pego a bola do topo
							if (standing) {
								// Mas primeiro tenho que me agachar
								return new Move("crouch");
							} else {
								// Marco boneco como semi-pronto, pois em seguida vou querer remontá-lo
								//ground[i][j] = Const.GROUND_LM;
								ground[i][j] = GROUND_TAKEN;
								blueSnowManDown = true;
								newSnowManPoint = new Point(i,j);
								return new Move("pickup",i,j);
							}
						} else {
							// Guardo posi��o do boneco mais pr�ximo
							if (dsq < dist) {
								dist = dsq;
								po = new Point(i,j);
							}
							// Vou em sua direção para poder acertar a bolada ou pegar a bola do topo
							//return moveToward(new Point(i,j));
						}
//						}
					}
				}
			}
			if (smb && po != null) {
				// Marca boneco escolhido pela crian�a para as outras n�o acharem
				ground[po.x][po.y] = GROUND_SMBT;
				if (!standing) {
					return new Move("stand");
				} else {
					return moveToward(po);
				}
			}
			
			// Visando a tática de atirar bolas
			// Se não tem nenhuma bola de neve pequena, tenta fazer uma
			if (holding != Const.HOLD_S1 && holding != Const.HOLD_S2 && holding != Const.HOLD_S3) {
	            // Se tenho neve, compacto em uma bola de neve
				if (holding == Const.HOLD_P1) {
					return new Move("crush");
				} else {
					// Não tenho neve, vejo se existe alguma nas proximidades
					int sx = -1, sy = -1;
					for (int ox = pos.x - 1; ox <= pos.x + 1; ox++)
						for (int oy = pos.y - 1; oy <= pos.y + 1; oy++) {
							// Existe neve para pegar?
							if (ox >= 0 
									&& ox < Const.SIZE 
									&& oy >= 0
									&& oy < Const.SIZE
									&& (ox != pos.x || oy != pos.y)
									&& ground[ox][oy] == Const.GROUND_EMPTY
									&& height[ox][oy] > 0) {
								sx = ox;
								sy = oy;
							}
						}

					// Se existe neve para pegar, tenta pegar.
					if (sx >= 0) {
						if (standing) {
							return new Move("crouch");
						} else {
							// Marco local como utilizado para que as outras crianças não tentem
							// pegar neve do mesmo local
							ground[sx][sy] = GROUND_TAKEN;
							return new Move("pickup", sx, sy);
						}
					}
				}
			} 
			
			// Se estou armado (com bolas pequenas), tento atirar no adversário mais próximo
			if ( holding == Const.HOLD_S1 || holding == Const.HOLD_S2 || holding == Const.HOLD_S3) {
				// Primeiro vejo se consigo atirar estando agachado
				if (!standing) {
					// tento encontrar uma vitima que atinja mesmo eu estando agachado
					// Variáveis para localização da melhor atirada
					boolean thro = false;
					Move to = null;
					dist = 99999;

					// Tento encontrar a vítima
					for (int j = Const.CCOUNT; j < Const.CCOUNT * 2; j++) {
						if (cList[j].pos.x >= 0 && cList[j].dazed <= 2) {
							int dx = cList[j].pos.x - pos.x;
							int dy = cList[j].pos.y - pos.y;
							int dsq = dx * dx + dy * dy;
							if (dsq < 8 * 8) {
								// throw past the victim, so we will probably
								// hit them before the snowball falls into the snow.
								// testa se tem obstaculos na trajetoria da bola
								double f = 3.0;
								Point victim = cList[j].pos;
								Point target = new Point(victim.x + (int)(f*dx), victim.y + (int)(f*dy));
								if (obstacleInPath(victim, target)) {
									//System.err.println("ha obstaculo: " + cList[j].pos.x + " " + cList[j].pos.y);
									continue;	// tem obstaculo, desconsidera essa vitima
								}
								thro = true;
								if (dsq < dist) {
									to = new Move("throw", pos.x + (int)(f*dx), pos.y + (int)(f*dy));
									dist = dsq;
								}
							}
						}
					}
					// Retorna o melhor arremesso
					if (thro) {
						return to;
					}
					else {
						// Nao consegui arremesar, levanto
						return new Move("stand");
					}
				} 
				else {	// estou armado e de peh
					// Variáveis para localização da melhor atirada
					boolean thro = false;
					Move to = null;
					dist = 99999;

					// Tento encontrar a vítima
					for (int j = Const.CCOUNT; j < Const.CCOUNT * 2; j++) {
						if (cList[j].pos.x >= 0 && cList[j].dazed <= 2) {
							int dx = cList[j].pos.x - pos.x;
							int dy = cList[j].pos.y - pos.y;
							int dsq = dx * dx + dy * dy;
							if (dsq < 15 * 15) {
								// throw past the victim, so we will probably
								// hit them before the snowball falls into the snow.
								// testa se tem obstaculos na trajetoria da bola
								double f = dsq < 12 * 12 ? 2.0 : 1.6;
								Point victim = cList[j].pos;
								Point target = new Point(victim.x + (int)(f*dx), victim.y + (int)(f*dy));
								if (obstacleInPath(victim, target)) {
									//System.err.println("ha obstaculo: " + cList[j].pos.x + " " + cList[j].pos.y);
									continue;	// tem obstaculo, desconsidera essa vitima
								}
								thro = true;
								if (dsq < dist) {
									to = new Move("throw", pos.x + (int)(f*dx), pos.y + (int)(f*dy));
									dist = dsq;
								}
							}
						}
					}
					// Retorna o melhor arremesso
					if (thro) {
						return to;
					}
				}
			}

			// TODO TESTAR Tento colocar uma bola de neve pequena em cima de um boneco quase pronto
			for (int i = 0; i < Const.SIZE; i++) {
				for (int j = 0; j < Const.SIZE; j++) {
					// Se existe um boneco semi-pronto
					if (ground[i][j] == Const.GROUND_LM) {
						int dx = i - pos.x;
						int dy = j - pos.y;
						int dsq = dx * dx + dy * dy;
						if (dsq <= 2 &&
							(holding == Const.HOLD_S1 || holding == Const.HOLD_S2 || holding == Const.HOLD_S3)) {
							// Estou do lado do boneco, coloco uma bola de neve pequena encima
							return new Move("drop",i,j);
						} else {
							// Vou em sua direção para poder colocar uma bola do topo
// TODO Escolher boneco mais próximo de cada criança para não irem 
// todas na direção do mesmo boneco
							if (!standing) {
								return new Move("stand");
							} else {
								return moveToward(new Point(i,j));
							}
						}
					}
				}
			}
			
			
			// Não está construindo um boneco de neve
//			if (state == 0) {
//
//				// If we didn't get to finish the last snowman, 
//				// maybe we're holding shomething.
//				// We should drop it.
//				if (holding != Const.HOLD_EMPTY && pos.y < Const.SIZE - 1
//						&& height[pos.x][pos.y + 1] <= Const.MAX_PILE - 3)
//					return new Move("drop", pos.x, pos.y + 1);
//
//				// Find the nearest neighbor.
//				int nearDist = 1000;
//				for (int i = 0; i < Const.SIZE; i++)
//					for (int j = 0; j < Const.SIZE; j++)
//						// Se não é a posição atual e
//						// existe uma criança ou um boneco vermelho (meu)
//						if ((i != pos.x || j != pos.y)
//								&& (ground[i][j] == GROUND_CHILD || ground[i][j] == Const.GROUND_SMR)) {
//
//							int dx = (pos.x - i);	// Pega as distâncias x e y
//							int dy = (pos.y - j);
//
//							// 
//							if (dx * dx + dy * dy < nearDist)
//								nearDist = dx * dx + dy * dy;
//						}
//
//				// See if we should start running our build script.
//				// Are we far from other things, is the ground empty
//				// and do we have enough snow to build a snowman.
//				if (nearDist > 5 * 5 
//						&& pos.x < Const.SIZE - 1
//						&& pos.y < Const.SIZE - 1
//						&& ground[pos.x + 1][pos.y] == Const.GROUND_EMPTY
//						&& ground[pos.x + 1][pos.y + 1] == Const.GROUND_EMPTY
//						&& height[pos.x + 1][pos.y] >= 3
//						&& height[pos.x + 1][pos.y + 1] >= 3
//						&& holding == Const.HOLD_EMPTY) {
//					// Start trying to build a snowman.
//					state = 1;
//				}
//			}

			// Está construindo um boneco de neve
//			if (state > 0) {
//				// Stamp out a move from our instruction template and return it.
//				Move m = new Move(instructions[state].action);
//				if (instructions[state].dest != null)
//					m.dest = new Point(pos.x + instructions[state].dest.x,
//									   pos.y + instructions[state].dest.y);
//				state = (state + 1) % instructions.length;
//
//				return m;
//			}

			
			// TODO Melhorar o esquema de movimentação das crianças,
			// definir trajetos fixos para as crianças???
			
			// See if the child needs a new, random destination.

			//se há uma criança tonta, corro para ajudá-la
			if(id==1){
				for (int j = 0; j < Const.CCOUNT; j++) {
					Child c = cList[j];		
					if(c.dazed >0){
					int dx = c.pos.x;
					int dy = c.pos.y;
					runTarget.setLocation(dx, dy);
					return moveToward(runTarget);
					}
					
				}
			}
			
		while (runTimer <= 0 || runTarget.equals(pos)) {
			// Pick somewhere to run, omit the top and righmost edges.
			runTarget.setLocation(rnd.nextInt(Const.SIZE - 1), rnd.nextInt(Const.SIZE - 1));
			runTimer = 1 + rnd.nextInt(14);
		}
		runTimer--;		

				
		
				


	

//			//Teste de localização fixa para as crianças
//			if (id == 0)
//				runTarget.setLocation(6, 8);
//			if (id == 1)
//				runTarget.setLocation(12, 6);
//			if (id == 2)
//				runTarget.setLocation(18, 5);
//			if (id == 3)
//				runTarget.setLocation(24, 4);
			//fazer uma criança andar em posiçoes fixas pelas laterais do campo e testar contra o sucker anterior
			
			return moveToward(runTarget);
		}
	}

	public void run() {
		
		// Cria as crianças
		for (int i = 0; i < cList.length; i++) {
			if (i < Const.CCOUNT)
				cList[i] = new SnowmanMaker();
			else
				cList[i] = new Child();

			// Atribui o id de cada criança
			cList[i].id = i;
		}

		// Scanner to parse input from the game engine.
		Scanner in = new Scanner(System.in);

		// Random destination for each player.
		Point[] runTarget = new Point[Const.CCOUNT];
		for (int i = 0; i < runTarget.length; i++)
			runTarget[i] = new Point();

		// How long the child has left to run toward its destination.
		// int[] runTimer = new int [ Const.CCOUNT ];

		// Keep reading states until the game ends.
		int turnNum = in.nextInt();
		while (turnNum >= 0) {
			this.turnNum = turnNum;
			String token;

			// Read current game score.
			score[Const.RED] = in.nextInt();
			score[Const.BLUE] = in.nextInt();

			// Parse the current map.
			for (int i = 0; i < Const.SIZE; i++) {
				for (int j = 0; j < Const.SIZE; j++) {
					// Can we see this cell?
					token = in.next();
					if (token.charAt(0) == '*') {
						height[i][j] = -1;
						ground[i][j] = -1;
					} else {
						height[i][j] = token.charAt(0) - '0';

						// Salva o conteúdo do mapa da rodada anterior
						ground_old[i][j] = ground[i][j];
						// Salva o novo conteúdo
						ground[i][j] = token.charAt(1) - 'a';
					}
				}
			}

			// Read the states of all the children.
			for (int i = 0; i < Const.CCOUNT * 2; i++) {
				Child c = cList[i];

				// Can we see this child?
				token = in.next();
				if (token.equals("*")) {
					c.pos.x = -1;
					c.pos.y = -1;
				} else {
					// Record the child's location.
					c.pos.x = Integer.parseInt(token);
					c.pos.y = in.nextInt();

					// Compute child color based on it's index.
					c.color = (i < Const.CCOUNT ? Const.RED : Const.BLUE);

					// Read the stance, what the child is holding and how much
					// longer he's dazed.
					token = in.next();
					c.standing = token.equals("S");

					token = in.next();
					c.holding_old = c.holding;	// Salva o que a criança segurava na rodada anterior
					c.holding = token.charAt(0) - 'a';

					c.dazed = in.nextInt();
				}
			}

			// Mark all the children in the map, so they are easy to
			// look up.
			for (int i = 0; i < Const.CCOUNT * 2; i++) {
				Child c = cList[i];
				if (c.pos.x >= 0) {
					ground[c.pos.x][c.pos.y] = GROUND_CHILD;
				}
			}

			// Decide what each child should do
			for (int i = 0; i < Const.CCOUNT; i++) {
				Move m = cList[i].chooseMove();

				/** Write out the child's move */
				if (m.dest == null) {
					System.out.println(m.action);
				} else {
					cList[i].moveOld = m.action;
					System.out.println(m.action + " " + m.dest.x + " "
							+ m.dest.y);
				}
			}

			turnNum = in.nextInt();
		}
	}

	public static void main(String[] args) {
		Sucker sucker = new Sucker();
		sucker.run();
	}
}
